interfaces

package
v0.0.0-...-9194542 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: May 6, 2026 License: Apache-2.0 Imports: 0 Imported by: 0

Documentation

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

This section is empty.

Types

type Cloner

type Cloner[T any] interface {
	Clone() T
}

Cloner is a generic interface for objects that can clone themselves.

type Iterator

type Iterator[T any] interface {
	// HasNext returns true if there are more elements to iterate.
	HasNext() bool

	// Next returns the next element in the iteration.
	Next() T
}

Iterator is a generic interface for iterators.

Jump to